package de.intevation.flys.artifacts.model.fixings;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallContext;

import de.intevation.flys.artifacts.FLYSArtifact;

import de.intevation.flys.artifacts.access.FixAnalysisAccess;

import de.intevation.flys.artifacts.math.fitting.Function;
import de.intevation.flys.artifacts.math.fitting.FunctionFactory;

import de.intevation.flys.artifacts.model.CalculationResult;
import de.intevation.flys.artifacts.model.DataFacet;
import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.model.Parameters;

import de.intevation.flys.artifacts.states.DefaultState.ComputeType;

import org.apache.log4j.Logger;

/**
 * Facet to show the W|Q values.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class FixWQCurveFacet
extends      DataFacet
implements   FacetTypes {

    /** House logger. */
    private static Logger logger = Logger.getLogger(FixWQCurveFacet.class);

    /** Trivial Constructor. */
    public FixWQCurveFacet() {
    }


    /**
     * @param description Description of the facet.
     */
    public FixWQCurveFacet(String description) {
        super(0, FIX_WQ_CURVE, description, ComputeType.ADVANCE, null, null);
    }

    public FixWQCurveFacet(int index, String description) {
        super(index, FIX_WQ_CURVE, description, ComputeType.ADVANCE, null, null);
    }


    /**
     * Returns the data this facet requires.
     *
     * @param artifact the owner artifact.
     * @param context  the CallContext (ignored).
     *
     * @return the data.
     */
    @Override
    public Object getData(Artifact artifact, CallContext context) {
        logger.debug("FixWQCurveFacet.getData");
        if (artifact instanceof FLYSArtifact) {
            FLYSArtifact flys = (FLYSArtifact)artifact;
            FixAnalysisAccess access = new FixAnalysisAccess(flys);

            CalculationResult res =
                (CalculationResult) flys.compute(context,
                                                 ComputeType.ADVANCE,
                                                 false);

            FixResult result = (FixResult) res.getData();
            double currentKm =
                ((Double)context.getContextValue("currentKm")).doubleValue();

            logger.debug("getData: km = " + currentKm);

            String function = access.getFunction();
            Function ff = FunctionFactory.getInstance().getFunction(function);

            if (ff == null) {
                logger.warn("getData: ff == null");
                return null;
            }

            Parameters params = result.getParameters();
            String[] paramNames = ff.getParameterNames();

            double [] coeffs = params.interpolate("km", currentKm, paramNames);

            if (coeffs == null) {
                logger.warn("getData: coeffs == null");
                return null;
            }

            de.intevation.flys.artifacts.math.Function mf =
                ff.instantiate(coeffs);

            double maxQ = FixFacetUtils.getMaxQ(params, currentKm);
            logger.debug("getData: maxQ = " + maxQ);

            FixFunction fix = new FixFunction(
                ff.getName(),
                ff.getDescription(),
                mf,
                maxQ);

            return fix;
        }
        else {
            logger.debug("Not an instance of FLYSArtifact / FixationArtifact.");
            return null;
        }
    }

    /**
     * Create a deep copy of this Facet.
     * @return a deep copy.
     */
    @Override
    public FixWQCurveFacet deepCopy() {
        FixWQCurveFacet copy = new FixWQCurveFacet();
        copy.set(this);
        return copy;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
